@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014, 2015 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -204,15 +204,15 @@ class Object(object):
 
     .. attribute:: xvelocity
 
-       The velocity of the object toward the right.
+       The velocity of the object toward the right in pixels per frame.
 
     .. attribute:: yvelocity
 
-       The velocity of the object toward the bottom.
+       The velocity of the object toward the bottom in pixels per frame.
 
     .. attribute:: speed
 
-       The total (directional) speed of the object.
+       The total (directional) speed of the object in pixels per frame.
 
     .. attribute:: move_direction
 
@@ -220,6 +220,30 @@ class Object(object):
        being directly to the right and rotation in a positive direction
        being counter-clockwise.
 
+    .. attribute:: xacceleration
+
+       The acceleration of the object to the right in pixels per frame.
+       If non-zero, movement as a result of :attr:`xvelocity` will be
+       adjusted based on the kinematic equation,
+       ``v[f]^2 = v[i]^2 + 2*a*d``.
+
+    .. attribute:: yacceleration
+
+       The acceleration of the object downward in pixels per frame.  If
+       non-zero, movement as a result of :attr:`yvelocity` will be
+       adjusted based on the kinematic equation,
+       ``v[f]^2 = v[i]^2 + 2*a*d``.
+
+    .. attribute:: xdeceleration
+
+       Like :attr:`xacceleration`, but its sign is ignored and it always
+       causes the absolute value of :attr:`xvelocity` to decrease.
+
+    .. attribute:: ydeceleration
+
+       Like :attr:`yacceleration`, but its sign is ignored and it always
+       causes the absolute value of :attr:`yvelocity` to decrease.
+
     .. attribute:: image_index
 
        The animation frame currently being displayed, with ``0`` being
@@ -690,9 +714,11 @@ class Object(object):
                  bbox_y=None, bbox_width=None, bbox_height=None,
                  regulate_origin=False, collision_ellipse=False,
                  collision_precise=False, xvelocity=0, yvelocity=0,
-                 image_index=0, image_origin_x=None, image_origin_y=None,
-                 image_fps=None, image_xscale=1, image_yscale=1,
-                 image_rotation=0, image_alpha=255, image_blend=None):
+                 xacceleration=0, yacceleration=0, xdeceleration=0,
+                 ydeceleration=0, image_index=0, image_origin_x=None,
+                 image_origin_y=None, image_fps=None, image_xscale=1,
+                 image_yscale=1, image_rotation=0, image_alpha=255,
+                 image_blend=None):
         """
         Arugments set the respective initial attributes of the object.
         See the documentation for :class:`sge.Object` for more
@@ -711,6 +737,10 @@ class Object(object):
         self.rd["yv"] = 0
         self.rd["mv_dir"] = 0
         self.rd["speed"] = 0
+        self.xacceleration = xacceleration
+        self.yacceleration = yacceleration
+        self.xdeceleration = xdeceleration
+        self.ydeceleration = ydeceleration
         self.__image_index = image_index
         self.image_origin_x = image_origin_x
         self.image_origin_y = image_origin_y
@@ -758,6 +788,40 @@ class Object(object):
         self.visible = visible
         self.image_fps = image_fps
 
+    def move_x(self, move):
+        """
+        Move the object horizontally.  This method can be overridden to
+        conveniently define a particular way movement should be handled.
+        Currently, it is used in the default implementation of
+        :meth:`event_update_position`.
+
+        Arguments:
+
+        - ``move`` -- The amount to add to :attr:`x`.
+
+        The default behavior of this method is the following code::
+
+            self.x += move
+        """
+        self.x += move
+
+    def move_y(self, move):
+        """
+        Move the object vertically.  This method can be overridden to
+        conveniently define a particular way movement should be handled.
+        Currently, it is used in the default implementation of
+        :meth:`event_update_position`.
+
+        Arguments:
+
+        - ``move`` -- The amount to add to :attr:`y`.
+
+        The default behavior of this method is the following code::
+
+            self.y += move
+        """
+        self.y += move
+
     def collision(self, other=None, x=None, y=None):
         """
         Return a list of objects colliding with this object.
@@ -996,86 +1060,62 @@ class Object(object):
         override the default behavior and allow you to handle the speed
         variables in a special way.
 
-        The default behavior of this method is::
+        The default behavior of this method is the following code::
 
-            self.x += self.xvelocity * delta_mult
-            self.y += self.yvelocity * delta_mult
+            if delta_mult:
+                vf = self.xvelocity + self.xacceleration * delta_mult
+                dc = min(abs(vf), abs(self.xdeceleration))
+                a = self.xacceleration - math.copysign(dc, vf)
+                self.move_x(self.xvelocity * delta_mult +
+                            0.5 * a * (delta_mult ** 2))
+                self.xvelocity += a * delta_mult
+
+                vf = self.yvelocity + self.yacceleration * delta_mult
+                dc = min(abs(vf), abs(self.ydeceleration))
+                a = self.yacceleration - math.copysign(dc, vf)
+                self.move_y(self.yvelocity * delta_mult +
+                            0.5 * a * (delta_mult ** 2))
+                self.yvelocity += a * delta_mult
 
         See the documentation for :meth:`sge.Game.event_step` for more
         information.
         """
-        self.x += self.xvelocity * delta_mult
-        self.y += self.yvelocity * delta_mult
+        if delta_mult:
+            vf = self.xvelocity + self.xacceleration * delta_mult
+            dc = min(abs(vf), abs(self.xdeceleration))
+            a = self.xacceleration - math.copysign(dc, vf)
+            self.move_x(self.xvelocity * delta_mult +
+                        0.5 * a * (delta_mult ** 2))
+            self.xvelocity += a * delta_mult
 
-    def event_collision(self, other):
+            vf = self.yvelocity + self.yacceleration * delta_mult
+            dc = min(abs(vf), abs(self.ydeceleration))
+            a = self.yacceleration - math.copysign(dc, vf)
+            self.move_y(self.yvelocity * delta_mult +
+                        0.5 * a * (delta_mult ** 2))
+            self.yvelocity += a * delta_mult
+
+    def event_collision(self, other, xdirection, ydirection):
         """
-        Called when another object collides with this object and none of
-        the directional collision events are appropriate.  In
-        particular, this is called if the collision was already
-        happening in the previous frame.  This is also the event method
-        called by the directional collision event methods by default.
+        Called when this object collides with another object.
 
         Arguments:
 
         - ``other`` -- The other object which was collided with.
+        - ``xdirection`` -- The horizontal direction of the collision
+          from the perspective of this object.  Can be ``-1`` (left),
+          ``1`` (right), or ``0`` (no horizontal direction).
+        - ``ydirection`` -- The vertical direction of the collision from
+          the perspective of this object.  Can be ``-1`` (up), ``1``
+          (down), or ``0`` (no vertical direction).
+
+        Directionless "collisions" (ones with both an xdirection and
+        ydirection of ``0``) are possible.  These are typically
+        collisions which were already occurring in the previous frame
+        (continuous collisions).
         """
         pass
 
-    def event_collision_left(self, other):
-        """
-        Called when another object collides with this object's left
-        side.
-
-        Arguments:
-
-        - ``other`` -- The other object which was collided with.
-
-        By default, this method simply calls
-        :meth:`sge.Object.event_collision`.
-        """
-        self.event_collision(other)
-
-    def event_collision_right(self, other):
-        """
-        Called when another object collides with this object's right
-        side.
-
-        Arguments:
-        
-        - ``other`` -- The other object which was collided with.
-
-        By default, this method simply calls
-        :meth:`sge.Object.event_collision`.
-        """
-        self.event_collision(other)
-
-    def event_collision_top(self, other):
-        """
-        Called when another object collides with this object's top side.
-
-        Arguments:
-
-        - ``other`` -- The other object which was collided with.
-
-        By default, this method simply calls
-        :meth:`sge.Object.event_collision`.
-        """
-        self.event_collision(other)
-
-    def event_collision_bottom(self, other):
-        """
-        Called when another object collides with this object's bottom
-        side.
-
-        Arguments:
-
-        - ``other`` -- The other object which was collided with.
-
-        By default, this method simply calls
-        :meth:`sge.Object.event_collision`.
-        """
-        self.event_collision(other)
-
     def event_inactive_step(self, time_passed, delta_mult):
         """
         See the documentation for :meth:`sge.Object.event_step`
@@ -1364,20 +1404,8 @@ class Mouse(Object):
     def event_step(self, time_passed, delta_mult):
         o_update_collision_lists(self)
 
-    def event_collision(self, other):
-        sge.game.event_mouse_collision(other)
-
-    def event_collision_left(self, other):
-        sge.game.event_mouse_collision_left(other)
-
-    def event_collision_right(self, other):
-        sge.game.event_mouse_collision_right(other)
-
-    def event_collision_top(self, other):
-        sge.game.event_mouse_collision_top(other)
-
-    def event_collision_bottom(self, other):
-        sge.game.event_mouse_collision_bottom(other)
+    def event_collision(self, other, xdirection, ydirection):
+        sge.game.event_mouse_collision(other, xdirection, ydirection)
 
     def set_cursor(self):
         # Set the mouse cursor and visibility state.
