@@ -1,4 +1,4 @@
-# Copyright (C) 2012, 2013, 2014 Julian Marchant <onpon4@riseup.net>
+# Copyright (C) 2012, 2013, 2014, 2015 Julian Marchant <onpon4@riseup.net>
 # 
 # This file is part of the Pygame SGE.
 # 
@@ -27,14 +27,13 @@ import sge
 from sge import r
 from sge.r import (_get_dot_sprite, _get_line_sprite, _get_rectangle_sprite,
                    _get_ellipse_sprite, _get_circle_sprite,
-                   _get_polygon_sprite, _get_text_sprite,
-                   o_update_collision_lists, r_reset, r_update_fade,
-                   r_update_dissolve, r_update_pixelate, r_update_wipe_left,
-                   r_update_wipe_right, r_update_wipe_up, r_update_wipe_down,
-                   r_update_wipe_upleft, r_update_wipe_upright,
-                   r_update_wipe_downleft, r_update_wipe_downright,
-                   r_update_wipe_matrix, r_update_iris_in, r_update_iris_out,
-                   s_get_image)
+                   _get_polygon_sprite, o_update_collision_lists,
+                   r_update_fade, r_update_dissolve, r_update_pixelate,
+                   r_update_wipe_left, r_update_wipe_right, r_update_wipe_up,
+                   r_update_wipe_down, r_update_wipe_upleft,
+                   r_update_wipe_upright, r_update_wipe_downleft,
+                   r_update_wipe_downright, r_update_wipe_matrix,
+                   r_update_iris_in, r_update_iris_out, s_get_image)
 
 
 __all__ = ['Room']
@@ -123,46 +122,19 @@ class Room(object):
             self.views = list(views)
         else:
             self.views = [sge.View(0, 0)]
-        self.rd["sviews"] = []
-
-        self.rd["view_sx"] = {}
-        self.rd["view_sy"] = {}
-        self.rd["view_sxport"] = {}
-        self.rd["view_syport"] = {}
-        self.rd["view_swidth"] = {}
-        self.rd["view_sheight"] = {}
 
         if background is not None:
             self.background = background
         else:
             self.background = sge.Background((), sge.Color("black"))
-        self.rd["sbackground"] = self.background
-        self.rd["sbackground_x"] = self.background_x
-        self.rd["sbackground_y"] = self.background_y
 
         self.rd["started"] = False
-        self.__has_started = False
 
         self.objects = []
 
         self.add(sge.game.mouse)
         for obj in objects:
             self.add(obj)
-        self.rd["sobjects"] = []
-
-        self.rd["object_sx"] = {}
-        self.rd["object_sy"] = {}
-        self.rd["object_sz"] = {}
-        self.rd["object_ssprite"] = {}
-        self.rd["object_svisible"] = {}
-        self.rd["object_schecks_collisions"] = {}
-        self.rd["object_stangible"] = {}
-        self.rd["object_sbbox_x"] = {}
-        self.rd["object_sbbox_y"] = {}
-        self.rd["object_sbbox_width"] = {}
-        self.rd["object_sbbox_height"] = {}
-        self.rd["object_scollision_ellipse"] = {}
-        self.rd["object_scollision_precise"] = {}
 
         if self.views:
             size = max(int(self.views[0].width / 10),
@@ -217,14 +189,16 @@ class Room(object):
             objects.remove(obj)
         self.objects = objects
 
+        while obj in self.__new_objects:
+            self.__new_objects.remove(obj)
+
         o_update_collision_lists(obj)
         if self is sge.game.current_room:
             obj.event_destroy()
 
     def start(self, transition=None, transition_time=1500):
         """
-        Start the room, resetting to its original state if it has been
-        started previously.
+        Start the room.
 
         Arguments:
 
@@ -253,17 +227,6 @@ class Room(object):
           milliseconds.  Has no effect if ``transition`` is
           :const:`None`.
         """
-        if self.__has_started:
-            r_reset(self)
-
-        self.resume(transition, transition_time)
-
-    def resume(self, transition=None, transition_time=1500):
-        """
-        Start the room without resetting to its original state if it has
-        been started previously.  See the documentation for
-        :meth:`sge.Room.start` for more information.
-        """
         transitions = {
             "fade": r_update_fade, "dissolve": r_update_dissolve,
             "pixelate": r_update_pixelate, "wipe_left": r_update_wipe_left,
@@ -289,31 +252,8 @@ class Room(object):
         sge.game.unpause()
         sge.game.current_room = self
 
-        if not self.__has_started:
-            self.rd["sobjects"] = self.objects[:]
-            for obj in self.objects:
-                self.rd["object_sx"][id(obj)] = obj.x
-                self.rd["object_sy"][id(obj)] = obj.y
-                self.rd["object_sz"][id(obj)] = obj.z
-                self.rd["object_ssprite"][id(obj)] = obj.sprite
-                self.rd["object_svisible"][id(obj)] = obj.visible
-                self.rd["object_schecks_collisions"][id(obj)] = obj.checks_collisions
-                self.rd["object_stangible"][id(obj)] = obj.tangible
-                self.rd["object_sbbox_x"][id(obj)] = obj.bbox_x
-                self.rd["object_sbbox_y"][id(obj)] = obj.bbox_y
-                self.rd["object_sbbox_width"][id(obj)] = obj.bbox_width
-                self.rd["object_sbbox_height"][id(obj)] = obj.bbox_height
-                self.rd["object_scollision_ellipse"][id(obj)] = obj.collision_ellipse
-                self.rd["object_scollision_precise"][id(obj)] = obj.collision_precise
-
-            self.rd["sviews"] = self.views[:]
-            for view in self.views:
-                self.rd["view_sx"][id(view)] = view.x
-                self.rd["view_sy"][id(view)] = view.y
-                self.rd["view_sxport"][id(view)] = view.xport
-                self.rd["view_syport"][id(view)] = view.yport
-                self.rd["view_swidth"][id(view)] = view.width
-                self.rd["view_sheight"][id(view)] = view.height
+        r._colliders = []
+        r._collision_checkers = []
 
         for obj in self.objects:
             obj.rd["collision_areas"] = []
@@ -321,7 +261,6 @@ class Room(object):
 
         started = self.rd["started"]
         self.rd["started"] = True
-        self.__has_started = True
         if not started:
             self.event_room_start()
             for obj in self.objects:
@@ -541,8 +480,8 @@ class Room(object):
         self.rd["projections"].append((img, x, y, z, blend_mode))
 
     def project_text(self, font, text, x, y, z, width=None, height=None,
-                    color=sge.Color("black"), halign=sge.ALIGN_LEFT,
-                    valign=sge.ALIGN_TOP, anti_alias=True):
+                    color=sge.Color("black"), halign="left",
+                    valign="top", anti_alias=True):
         """
         Project text onto the room.
 
@@ -561,25 +500,24 @@ class Room(object):
             e = "`{}` is not a sge.Color object.".format(repr(color))
             raise TypeError(e)
 
-        sprite = _get_text_sprite(font, text, width, height, color, halign,
-                                  valign, anti_alias)
+        sprite = sge.Sprite.from_text(font, text, width, height, color, halign,
+                                      valign, anti_alias)
         self.project_sprite(sprite, 0, x, y, z)
 
     def event_room_start(self):
         """
-        Called when the room starts for the first time, and when it is
-        restarted with :meth:`sge.Room.start`.  It is always called
-        after any :meth:`sge.Game.event_game_start` and before any
-        :class:`sge.Object.event_create` occurring at the same time.
+        Called when the room is started for the first time.  It is
+        always called after any :meth:`sge.Game.event_game_start` and
+        before any :class:`sge.Object.event_create` occurring at the
+        same time.
         """
         pass
 
     def event_room_resume(self):
         """
-        Called when the room is resumed with :meth:`sge.Room.resume`
-        after it has already previously been started.  It is always
-        called before any :meth:`sge.Object.event_create` occurring at
-        the same time.
+        Called when the room is started after it has already previously
+        been started.  It is always called before any
+        :meth:`sge.Object.event_create` occurring at the same time.
         """
         pass
 
@@ -600,7 +538,7 @@ class Room(object):
 
     def event_alarm(self, alarm_id):
         """
-        See the documentation for :meth:`sge.Room.alarms` for more
+        See the documentation for :attr:`sge.Room.alarms` for more
         information.
         """
         pass
@@ -715,6 +653,13 @@ class Room(object):
         """
         pass
 
+    def event_paused_step(self, time_passed, delta_mult):
+        """
+        See the documentation for :meth:`sge.Game.event_step` for more
+        information.
+        """
+        pass
+
     def event_paused_key_press(self, key, char):
         """
         See the documentation for :class:`sge.input.KeyPress` for more
