@@ -306,6 +306,37 @@ No other drawing or projecting methods support anti-aliasing.
 :data:`sge.BLEND_RGBA_SCREEN` and :data:`sge.BLEND_RGB_SCREEN` are
 unsupported. If one of these blend modes is attempted, normal blending
 will be used instead.
+
+Known Problems
+--------------
+
+There is a bug in either Pygame or SDL, most likely SDL, which sometimes
+causes keyboard input to stop working.  In Pygame programs such as this
+one, this occurs when pygame.display.set_mode is called multiple times,
+which in the Pygame SGE occurs any time either the size of the window or
+the video mode (windowed or fullscreen) changes.  See this post from the
+SGE blog for more information:
+
+https://savannah.nongnu.org/forum/forum.php?forum_id=8113
+
+You may also be interested in this report on the Pygame issue tracker:
+
+https://bitbucket.org/pygame/pygame/issue/212/
+
+As mentioned in the post on the SGE blog, this is a particularly serious
+problem for anyone using the X Window System (e.g. pretty much any
+GNU/Linux user), or any other window system that gives complete control
+to fullscreen SDL applications.  On these systems, if the game requires
+keyboard input to either leave fullscreen or exit, the system will
+become unresponsive to everything that isn't sent directly to the kernel
+(such as the magic SysRq key in Linux systems).
+
+Luckily, the bug doesn't seem to affect mouse input, so if you allow the
+player to enter fullscreen mode in-game, it is highly recommended for
+you to provide some method of either exiting the game or exiting
+fullscreen with the mouse.  This can be a button somewhere on the screen
+if the game uses the mouse cursor, or it can be a simple mouse button
+click otherwise.
 """
 
 from __future__ import division
@@ -313,7 +344,7 @@ from __future__ import absolute_import
 from __future__ import print_function
 from __future__ import unicode_literals
 
-__version__ = "0.16"
+__version__ = "0.17"
 
 import sys
 import os
