@@ -28,6 +28,7 @@ from __future__ import unicode_literals
 
 import math
 import random
+import time
 import warnings
 
 import pygame
@@ -36,8 +37,12 @@ import six
 import sge
 
 
+# How large the squares that make up the matrix transition are.
 MATRIX_PSIZE = 4
 
+# How often to call cache.prune in milliseconds.
+CACHE_PRUNE_TIME = 8000
+
 # Lists of objects that are tangible and objects that check for
 # collisions; makes collision detection more efficient.
 _colliders = []
@@ -46,18 +51,42 @@ _collision_checkers = []
 
 class cache(object):
 
-    dot_sprite = {}
-    line_sprite = {}
-    rect_sprite = {}
-    ellipse_sprite = {}
-    circle_sprite = {}
-    poly_sprite = {}
-    text_sprite = {}
+    prune_time = 0
+    _cache = {}
+    _prune = {}
 
-    rectangle_masks = {}
-    ellipse_masks = {}
-    circle_masks = {}
-    line_masks = {}
+    @classmethod
+    def add(cls, i, value, prune_time=30):
+        # Add value with index ``i`` to cache as ``value``.
+        # Automatically deleted after ``prune_time`` seconds.
+        cls._cache[i] = value
+        cls._prune[i] = time.time() + prune_time
+
+    @classmethod
+    def get(cls, i):
+        # Get value with index ``i`` from cache, or ``None`` if it is
+        # not in the cache.
+        return cls._cache.get(i)
+
+    @classmethod
+    def clear(cls):
+        # Clear all saved values from the cache.
+        cls._cache = {}
+        cls._prune = {}
+
+    @classmethod
+    def prune(cls):
+        # Prune all expired values. Should be called infrequently (maybe
+        # every 5 seconds or so), because it requires looping through
+        # every cached value.
+        p = []
+        for i in cls._prune:
+            if cls._prune[i] < time.time():
+                p.append(i)
+
+        for i in p:
+            del cls._cache[i]
+            del cls._prune[i]
 
 
 def _scale(surface, width, height):
@@ -225,14 +254,13 @@ def _handle_music():
 
 def _get_dot_sprite(color):
     # Return a sprite for the given dot.
-    i = tuple(color)
-    if i in cache.dot_sprite:
-        sprite = cache.dot_sprite[i]
-    else:
+    i = ("dot_sprite", tuple(color))
+    sprite = cache.get(i)
+    if sprite is None:
         sprite = sge.Sprite(None, width=1, height=1)
         sprite.draw_dot(0, 0, color)
-        cache.dot_sprite[i] = sprite
 
+    cache.add(i, sprite)
     return sprite
 
 
@@ -240,25 +268,24 @@ def _get_line_sprite(x1, y1, x2, y2, color, thickness, anti_alias):
     # Return a sprite for the given line.
     w = int(round(abs(x2 - x1) + thickness))
     h = int(round(abs(y2 - y1) + thickness))
-    i = (x1, y1, x2, y2, tuple(color), thickness, anti_alias)
-    if i in cache.line_sprite:
-        sprite = cache.line_sprite[i]
-    else:
+    i = ("line_sprite", x1, y1, x2, y2, tuple(color), thickness, anti_alias)
+    sprite = cache.get(i)
+    if sprite is None:
         sprite = sge.Sprite(None, width=w, height=h)
         sprite.draw_line(x1, y1, x2, y2, color, thickness, anti_alias)
-        cache.line_sprite[i] = sprite
 
+    cache.add(i, sprite)
     return sprite
 
 
 def _get_rectangle_sprite(width, height, fill, outline, outline_thickness):
     # Return a sprite for the given rectangle.
-    i = (width, height, tuple(fill) if fill is not None else None,
+    i = ("rectangle_sprite", width, height,
+         tuple(fill) if fill is not None else None,
          tuple(outline) if outline is not None else None,
          outline_thickness)
-    if i in cache.rect_sprite:
-        sprite = cache.rect_sprite[i]
-    else:
+    sprite = cache.get(i)
+    if sprite is None:
         outline_thickness = abs(outline_thickness)
         draw_x = outline_thickness // 2
         draw_y = outline_thickness // 2
@@ -267,20 +294,20 @@ def _get_rectangle_sprite(width, height, fill, outline, outline_thickness):
         sprite = sge.Sprite(None, width=w, height=h)
         sprite.draw_rectangle(draw_x, draw_y, width, height, fill, outline,
                               outline_thickness)
-        cache.rect_sprite[i] = sprite
 
+    cache.add(i, sprite)
     return sprite
 
 
 def _get_ellipse_sprite(width, height, fill, outline, outline_thickness,
                         anti_alias):
     # Return a sprite for the given ellipse.
-    i = (width, height, tuple(fill) if fill is not None else None,
+    i = ("ellipse_sprite", width, height,
+         tuple(fill) if fill is not None else None,
          tuple(outline) if outline is not None else None,
          outline_thickness, anti_alias)
-    if i in cache.ellipse_sprite:
-        sprite = cache.ellipse_sprite[i]
-    else:
+    sprite = cache.get(i)
+    if sprite is None:
         outline_thickness = abs(outline_thickness)
         draw_x = outline_thickness // 2
         draw_y = outline_thickness // 2
@@ -289,38 +316,37 @@ def _get_ellipse_sprite(width, height, fill, outline, outline_thickness,
         sprite = sge.Sprite(None, width=w, height=h)
         sprite.draw_ellipse(draw_x, draw_y, width, height, fill, outline,
                             outline_thickness)
-        cache.ellipse_sprite[i] = sprite
 
+    cache.add(i, sprite)
     return sprite
 
 
 def _get_circle_sprite(radius, fill, outline, outline_thickness, anti_alias):
     # Return a sprite for the given circle.
-    i = (radius, tuple(fill) if fill is not None else None,
+    i = ("circle_sprite", radius, tuple(fill) if fill is not None else None,
          tuple(outline) if outline is not None else None,
          outline_thickness, anti_alias)
-    if i in cache.circle_sprite:
-        sprite = cache.circle_sprite[i]
-    else:
+    sprite = cache.get(i)
+    if sprite is None:
         outline_thickness = abs(outline_thickness)
         xy = radius + outline_thickness // 2
         wh = 2 * radius + outline_thickness
         sprite = sge.Sprite(None, width=wh, height=wh)
         sprite.draw_circle(xy, xy, radius, fill, outline, outline_thickness,
                            anti_alias)
-        cache.circle_sprite[i] = sprite
 
+    cache.add(i, sprite)
     return sprite
 
 
 def _get_polygon_sprite(points, fill, outline, outline_thickness, anti_alias):
     # Return a sprite for the given rectangle.
-    i = (tuple(points), tuple(fill) if fill is not None else None,
+    i = ("poly_sprite", tuple(points),
+         tuple(fill) if fill is not None else None,
          tuple(outline) if outline is not None else None,
          outline_thickness, anti_alias)
-    if i in cache.poly_sprite:
-        sprite = cache.poly_sprite[i]
-    else:
+    sprite = cache.get(i)
+    if sprite is None:
         xlist = []
         ylist = []
         for point in points:
@@ -341,8 +367,8 @@ def _get_polygon_sprite(points, fill, outline, outline_thickness, anti_alias):
                             origin_y=draw_y)
         sprite.draw_polygon(dpoints, fill, outline, outline_thickness,
                             anti_alias)
-        cache.poly_sprite[i] = sprite
 
+    cache.add(i, sprite)
     return sprite
 
 
@@ -591,23 +617,8 @@ def o_detect_collisions(self):
             else:
                 ydirection = 0
 
-            if xdirection or ydirection:
-                if xdirection == 1:
-                    self.event_collision_right(other)
-                    other.event_collision_left(self)
-                elif xdirection == -1:
-                    self.event_collision_left(other)
-                    other.event_collision_right(self)
-
-                if ydirection == 1:
-                    self.event_collision_bottom(other)
-                    other.event_collision_top(self)
-                elif ydirection == -1:
-                    self.event_collision_top(other)
-                    other.event_collision_bottom(self)
-            else:
-                self.event_collision(other)
-                other.event_collision(self)
+            self.event_collision(other, xdirection, ydirection)
+            other.event_collision(self, -xdirection, -ydirection)
 
 
 def o_get_image_width(self):
@@ -897,13 +908,10 @@ def s_set_size(self):
 
 
 def s_refresh(self):
-    # Set the _images list based on the variables.
+    # Forget any cached values by moving to the next draw cycle.
     if not self.rd["locked"]:
-        self.rd["images"] = []
-        for image in self.rd["baseimages"]:
-            img = s_set_transparency(self, image)
-            img = _scale(img, self.width, self.height)
-            self.rd["images"].append({(1, 1, 0, 255, None): img})
+        self.rd["drawcycle"] += 1
+        self.rd["drawcycle"] %= 999999999999999
 
 
 def s_set_transparency(self, image):
@@ -924,13 +932,11 @@ def s_set_transparency(self, image):
 def s_get_image(self, num, xscale=1, yscale=1, rotation=0, alpha=255,
                 blend=None):
     num %= self.frames
-    # Return the properly sized surface.
-    t = (xscale, yscale, rotation, alpha,
-         tuple(blend) if blend is not None else blend)
-    if t in self.rd["images"][num]:
-        return self.rd["images"][num][t]
-    else:
-        # Hasn't been scaled to this size yet
+
+    i = ("s_image", self, self.rd["drawcycle"], num, xscale, yscale, rotation,
+         alpha, tuple(blend) if blend is not None else None)
+    img = cache.get(i)
+    if img is None:
         if xscale != 0 and yscale != 0:
             img = s_set_transparency(self, self.rd["baseimages"][num])
             xflip = xscale < 0
@@ -957,17 +963,17 @@ def s_get_image(self, num, xscale=1, yscale=1, rotation=0, alpha=255,
             img = pygame.Surface((1, 1))
             img.set_colorkey((0, 0, 0), pygame.RLEACCEL)
 
-        self.rd["images"][num][t] = img
-        return img
+    cache.add(i, img)
+    return img
 
 
 def s_get_precise_mask(self, num, xscale, yscale, rotation):
     # Return a precise mask (2D list of True/False values) for the
     # given image index.
-    mask_id = (num, xscale, yscale, rotation)
-    if mask_id in self.rd["masks"]:
-        return self.rd["masks"][mask_id]
-    else:
+    i = ("s_mask", hash(self), self.width, self.height, self.rd["drawcycle"],
+         num, xscale, yscale, rotation)
+    mask = cache.get(i)
+    if mask is None:
         image = s_set_transparency(self, self.rd["baseimages"][num])
         xflip = xscale < 0
         yflip = yscale < 0
@@ -991,10 +997,10 @@ def s_get_precise_mask(self, num, xscale, yscale, rotation):
                 mask.append([])
                 for y in six.moves.range(image.get_height()):
                     mask[x].append(image.get_at((x, y)) == colorkey)
-
         image.unlock()
-        self.rd["masks"][mask_id] = mask
-        return mask
+
+    cache.add(i, mask)
+    return mask
 
 
 def v_limit(self):
